/*

 Copyright (c) 2013 Texas A&M University-Corpus Christi. All rights reserved.
 Use of this source code is governed by the GPL version 3 (or, at your
 discretion, any later version.) A copy of the full license can be found in the
 LICENSE file.

 // Inspired by Paul Truong of Monocubed.com

 Library used:
    Google Maps API (http://code.google.com/apis/maps/documentation/javascript/)

 Methods from libraries:
    Google Maps: google.maps.OverlayView
 */

/** @constructor */
function Overlay(map, position, fields, opacity)
{
    // Set variables passed-in from arguments
    this.position_ = position;
    this.fields_ = fields;
    this.opacity_ = opacity;

    // Set canvas variables
    this.canvas_ = null;
    this.defWidth_ = 180;
    this.defHeight_ = 180;
    this.width_ = this.defWidth_;
    this.height_ = this.defHeight_;
    
    if(typeof pixelRatio === 'undefined') {
        pixelRatio = getPixelRatio();
    }

    // Adjust canvas width and store old canvas width
    if(pixelRatio != 1.0) {
        console.log("Adjusting ratio to " + pixelRatio);
        
        // Scale canvas width and height if pixel is not 1.0
        this.width_ = this.width_ * pixelRatio;
        this.height_ = this.height_ * pixelRatio;
    }

    // Set this overlay to the map, required for OverlayView
    this.setMap(map);
}

Overlay.prototype = new google.maps.OverlayView();

// Note: an overlay's receipt of onAdd() indicates that the map's panes are now
// available for attaching the overlay to the map via the DOM.
Overlay.prototype.onAdd = function() {
    // Create the canvas and set some basic attributes.
    var canvas = document.createElement("canvas");

    // Use modified width and height
    canvas.width = this.width_;
    canvas.height = this.height_;
    canvas.style.width = this.defWidth_ + 'px';
    canvas.style.height = this.defHeight_ + 'px';
    canvas.style.position = "absolute";
    canvas.style.zIndex = 5;

    // We add an overlay to a map via one of the map's panes.
    // We'll add this overlay to the overlayLayer pane.
    // Important! Setting to lower pane might decrease overlay visibility
    var panes = this.getPanes();
    panes.overlayImage.appendChild(canvas);

    // Set the overlay's canvas_ property to this canvas
    this.canvas_ = canvas;
}

Overlay.prototype.draw = function() {
    // console.log(window.innerWidth + ',' + window.innerHeight);

    var canvasPosition = this.getProjection().fromLatLngToDivPixel(this.position_);
    
    this.canvas_.style.left = canvasPosition.x  - this.defWidth_/2 + "px";
    this.canvas_.style.top = canvasPosition.y - this.defHeight_/2 + "px";

    // For debugging purposes
    // this.canvas_.style.border = "1px solid red";
    // this.canvas_.style.backgroundColor = "#FFF";

    var centerX = this.defWidth_ / 2;
    var centerY = this.defHeight_ / 2;

    // Initialize 2D context
    var context = this.canvas_.getContext("2d");

    this.clear();
    
    context.save();

    // Now scale the context to counter the fact that we've manually scaled
    // our canvas element
    context.scale(pixelRatio, pixelRatio);

    // Translate to middle of canvas
    context.translate(centerX, centerY);

    // // Arc variable
    var arcRadius = 16; // Set to 16 to have same size as default map markers

    // // Arrow variable
    var speedRatio = this.fields_.windSpeed.currentSpeed / this.fields_.windSpeed.greatestSpeed;
    if(speedRatio > 1.0) {
        speedRatio = 1.0;
    }
    var arrowColor = 'rgba(38,38,92, ' + this.opacity_ + ')';
    var arcColor = 'rgba(44,44,140, ' + this.opacity_ + ')';
    var textColor = 'white';

    var arrowDirection = convertAzimuthToRadian(this.fields_.wdr) * -1;

    // // Begin drawing inside canvas
    drawArc(context, 0, 0, arcRadius, arcColor);
    drawText(context, this.fields_.text, 0, 0, textColor);
    drawArrow(context, centerX, arcRadius, arrowDirection, speedRatio , arrowColor);

    context.restore();
}

// onRemove() method to cleanly remove the overlay from the map
Overlay.prototype.onRemove = function() {
    this.canvas_.parentNode.removeChild(this.canvas_);
}

Overlay.prototype.clear = function() {
    if (this.canvas_.getContext) {
        this.canvas_.getContext("2d").clearRect(0, 0, this.canvas_.width, this.canvas_.height);
    }
}

function drawArc(context, centerX, centerY, radius, arcColor) {
    context.save();

    context.beginPath();
    context.arc(centerX, centerY, radius, 0, 2 * Math.PI, false);
    context.fillStyle = arcColor;
    context.fill();
    context.lineWidth = 2;
    context.strokeStyle = '#1C2564';
    context.stroke();

    context.restore();
}

function drawText(context, text, x, y, textColor) {
    // Replace _MPH or _m/s or _kt with ""
    // gi for case-insensitive
    text = text.replace(/ mph| m\/s| kt/gi, "");
    
    context.save();

    context.fillStyle = textColor;
    context.font = "8pt Helvetica, sans-serif";
    context.textAlign = "center";
    context.textBaseline = "middle";
    context.fillText(text, x, y);

    context.restore();
}

function drawArrow(context, canvasRadius, centerOffset, direction, speedRatio, color) {
    // Disable wind arrow if wind magnitude is 0 MPH
    if(speedRatio == 0.0) {
        return;
    }

    //The triangle "tip" of the arrow's length
    var tipBaseToPoint = 15;
    // Arrow width
    var width = 5;
    // Arrow length
    var length = speedRatio * (canvasRadius - tipBaseToPoint - centerOffset);

    context.save();

    context.rotate(direction);
    context.translate(centerOffset, 0);

    // Begin drawing
    context.beginPath();
    context.fillStyle = color;
    context.fillRect(0, -width/2, length, width);
    context.moveTo(length, width/2);
    context.lineTo(length, width*3/2 );
    context.lineTo(length + tipBaseToPoint, 0);
    context.lineTo(length, -width*3/2);
    context.fill();

    context.restore();
}

function convertAzimuthToRadian(degrees)
{
    return (270 - degrees) * (Math.PI / 180);
}

function getPixelRatio()
{
    // Create the canvas and set some basic attributes.
    var canvas = document.createElement("canvas");

    // Check if canvas is supported
    if(canvas.getContext && canvas.getContext('2d'))
    {
        var context = canvas.getContext('2d');

        var devicePixelRatio = window.devicePixelRatio || 1;
        var backingStoreRatio = context.webkitBackingStorePixelRatio ||
                                context.mozBackingStorePixelRatio ||
                                context.msBackingStorePixelRatio ||
                                context.oBackingStorePixelRatio ||
                                context.backingStorePixelRatio || 1;

        var ratio = devicePixelRatio / backingStoreRatio;

        // ensure we have a value set for auto.
        // If auto is set to false then we
        // will simply not upscale the canvas
        // and the default behaviour will be maintained
        if (typeof auto === 'undefined') {
            auto = false;
        }

        // upscale the canvas if the two ratios don't match
        if (auto && ratio > 1.0) {
            return 2;
        }
    }

    // Return default pixel ratio
    return 1.0;
}
